# CVS $Id: FaceImportObject.py,v 1.1.1.1 2006/02/26 15:54:20 jay Exp $
#=========================================================================
# Process a f tag from the poser or daz .obj file
# 
#  Copyright (C) 2006 Jay van der Meer
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#=========================================================================
import string

class FaceImportObject:
	"""Parse a f record from a text obj file.  The constructor expects the raw data from the f record."""
	def __init__(self):
		self.MyFaceData = ''
		self.currentidx = 0
		self.datalen = 0
		self.novt = True
			
	def hasMore(self):
		""" If there are still more data groups return true"""
		if self.currentidx < self.datalen:
			return True
		else:
			return False

	def SetFaceDataRecord(self,FaceData):
		""" Split the four face data groups then determin if there is vertex text data or not.
			the expected format is v1 v2 v3 v4
			or v/vt/vn v/vt/vn v/vt/vn v/vt/vn
			or v//vn v//vn v//vn v//vn
			
			The DAZ format of v/vt is not acceptable, it needs to be exported with
			vertex normals to be correct.
		"""
		self.MyFaceData = FaceData[1:]
		self.currentidx = 0
		self.datalen = len(self.MyFaceData)
		
		if string.find(self.MyFaceData[0],'//') != -1:
			self.novt = True
		else:
			self.novt = False
			
	def next(self):
		""" Get the next data group. Return a list where:
			ResultList[0] = v
			ResultList[1] = vt
			ResultList[2] = vn
			
			If there is no vt information put a zero in ResultList[1]
		"""
		ResultList = []
		
		if self.currentidx < self.datalen:
			WorkData = string.split(self.MyFaceData[self.currentidx],'/')
			ResultList.append(int(WorkData[0]))
			
			if self.novt == False:
				ResultList.append(int(WorkData[1]))
			else:
				ResultList.append(0)
				
			ResultList.append(int(WorkData[2]))
			self.currentidx = self.currentidx + 1
			
		else:
			raise StopIteration

		return ResultList
