# CVS $Id: FaceImporter.py,v 1.3 2006/04/08 16:20:54 jay Exp $
#=========================================================================
# Processes all the f tags from the .obj file
# 
#  Copyright (C) 2006 Jay van der Meer
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#=========================================================================
import Blender
import os
from FaceImportObject import *
from PoserImporter import *

class FaceImporter:
	"""Handles the import of a type f record in a obj text file"""
		
	def __init__(self):
		self.MyFace = FaceImportObject()
		self.VTList = []
		self.VNList = []
		self.VertexLines = []
		self.VertexObjectList = {}
		
	def ProcessFaceRecordMT(self,TheFaceData,BlenderMeshObj):
		"""This method is for used when the importer only wants to deal with a Morph Target. The
			UV mapping (vt) lines are not created or used by this method.
			
			TheFace data should be the content of the f record.  This method will use
			FaceImportObject to parse the line then return the Vertex index, Vertex Text index
			and the Vertex normal index each as a list.
			
			A populated Blender Face object is returned
		"""
		VertIdxLst = []
		VertTxIdxLst = []
		VertNormalIdxLst = []
		FaceVertLst = []
		vertcounter = 0
		
		self.MyFace.SetFaceDataRecord(TheFaceData)
		
		while self.MyFace.hasMore() == True:
			WorkLst = self.MyFace.next()
			VertIdxLst.append(WorkLst[0])
			VertTxIdxLst.append(WorkLst[1])
			VertNormalIdxLst.append(WorkLst[0])
			
		for idx in VertIdxLst:
			vertcounter = vertcounter + 1
			
			if vertcounter > 4:
				print("WARNING:More than 4 vertices found for face:%s") % VertIdxLst
				break
			else:
				FaceVertLst.append(BlenderMeshObj.verts[idx-1])
			
		BlenderFace = Blender.NMesh.Face(FaceVertLst)
		
		return BlenderFace

	def ProcessFaceRecord(self,TheFaceData,BlenderMeshObj):
		"""Create a Face object with UV mapping (vt) records are used in this method.
		
			TheFace data should be the content of the f record.  This method will use
			FaceImportObject to parse the line then return the Vertex index, Vertex Text index
			and the Vertex normal index each as a list.
			
			A populated Blender Face object is returned
		"""
		VertIdxLst = []
		VertTxIdxLst = []
		VertNormalIdxLst = []
		FaceVertLst = []
		vertcounter = 0
		BlenderFace = None
		
		self.MyFace.SetFaceDataRecord(TheFaceData)
		
		while self.MyFace.hasMore() == True:
			WorkLst = self.MyFace.next()
			VertIdxLst.append(WorkLst[0])
			VertTxIdxLst.append(WorkLst[1])
			VertNormalIdxLst.append(WorkLst[0])
			
		for idx in VertIdxLst:
			vertcounter = vertcounter + 1
			
			if vertcounter > 4:
				print("WARNING:More than 4 vertices found for face:%s") % VertIdxLst
				break
			else:
				FaceVertLst.append(BlenderMeshObj.verts[idx-1])
			
		if vertcounter <= 4:
			BlenderFace = Blender.NMesh.Face(FaceVertLst)
		
			for idx in VertTxIdxLst:
				if idx != 0:
					BlenderFace.uv.append(self.VTList[idx-1])
		
		return BlenderFace
