# CVS $Id: MTLObject.py,v 1.2 2007/08/12 01:17:28 jay Exp $
#=========================================================================
# Object to parse and create the blender materials based on the .mtl file.
# 
#  Copyright (C) 2006 Jay van der Meer
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#=========================================================================
import Blender
import os
import string
import sys

class MTLObject:
	"""
		MTLObject represents the MTL file. It contains the methods needed to parse the mtl file and then
		serves an container for the Blender Materials.
		
	"""
	
	def __init__(self,FullPathName):
		self.MaterialObjectHash = {}
		self.MTLFilePath = FullPathName
	
	def ParseMTLFile(self,TheMTLFileFN):
		"""
			This method will parse the mtl file and build blender Material and Texture objects.
			
		"""
		
		TheCurrentMaterial = ''
		TheCurrentMatName = ''
		MyMTLFile = file(TheMTLFileFN,'r')
		
		for TheLine in MyMTLFile.readlines():
			WorkLine = string.strip(TheLine,'\r\n')
			WorkLineList = string.split(WorkLine)
			
			if WorkLineList[0] == "#":
				continue
			
			if WorkLineList[0] == "newmtl":
				
				if TheCurrentMaterial != None:
					self.MaterialObjectHash[TheCurrentMatName] = TheCurrentMaterial
				
				TheCurrentMatName = WorkLineList[1]
				TheCurrentMaterial = Blender.Material.New(TheCurrentMatName)
				
				print "Created new material %s" % TheCurrentMatName
				
			if WorkLineList[0] == "Ns":
				TheCurrentMaterial.setHardness(int(float(WorkLineList[1])))
				
			if WorkLineList[0] == 'Ka':
				TheCurrentMaterial.setMirCol(float(WorkLineList[1]),float(WorkLineList[2]),float(WorkLineList[3]))
				
			if WorkLineList[0] == 'Kd':
				TheCurrentMaterial.setRGBCol(float(WorkLineList[1]),float(WorkLineList[2]),float(WorkLineList[3]))
				
			if WorkLineList[0] == 'Ks':
				TheCurrentMaterial.setSpecCol(float(WorkLineList[1]),float(WorkLineList[2]),float(WorkLineList[3]))
				
			if WorkLineList[0] == 'd':
				workval = float(WorkLineList[1])
				TheCurrentMaterial.setAlpha(workval)
				
				if workval < 1:
					print("Setting ZTransp and Shadow")
					TheCurrentMaterial.mode |= Blender.Material.Modes.ZTRANSP
					TheCurrentMaterial.mode |= Blender.Material.Modes.SHADOW
				
			if WorkLineList[0] == 'Tr':
				TheCurrentMaterial.setAlpha(float(WorkLineList[1]))
				
			if WorkLineList[0] == 'map_Kd':
				MyImageFN = string.join(WorkLineList[1:],' ')
				MyTexture = self.__CreateImageTypeTexture(TheCurrentMatName + "_Kd",TheCurrentMaterial,MyImageFN)
				TheCurrentMaterial.setTexture(0,MyTexture,Blender.Texture.TexCo.UV,Blender.Texture.MapTo.COL)
			
			if WorkLineList[0] == 'map_d' or WorkLineList[0] == 'map_D':
				MyImageFN = string.join(WorkLineList[1:],' ')
				MyTexture = self.__CreateImageTypeTexture(TheCurrentMatName + "_d",TheCurrentMaterial,MyImageFN)
				TheCurrentMaterial.setTexture(1,MyTexture,Blender.Texture.TexCo.UV,Blender.Texture.MapTo.ALPHA)
				
		self.MaterialObjectHash[TheCurrentMatName] = TheCurrentMaterial
		MyMTLFile.close()
		
	def __CreateImageTypeTexture(self,TheCurrentMatName,TheCurrentMaterial,ImageFN):
		""" 
			Private method to create a Blender Texture image object, and load the requested
			image.
		"""
	
		TheTexture = Blender.Texture.New(TheCurrentMatName)
		TheTexture.setType('Image')
		
		if ImageFN.startswith('/') == False:
			ImagePath = self.MTLFilePath + os.sep + ImageFN
		else:
			ImagePath = self.MTLFilePath + ImageFN
			
		print(ImagePath)
		TheImage = Blender.Image.Load(ImagePath)
				
		if TheImage != None:
			print("Loaded image %s as material %s") % (ImagePath,TheCurrentMatName)
			TheTexture.setImage(TheImage)
		else:
			print("Failed to load image %s") % (ImagePath)
			
		return TheTexture
