# CVS $Id: MTLReWriter.py,v 1.5 2006/08/11 00:16:34 jay Exp $
#=========================================================================
# Provides methods to parse a .mtl file and the collapse file.
# 
#  Copyright (C) 2006 Jay van der Meer
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#=========================================================================
import os
import string
import sys
from MTLContainer import *

class MTLReWriter:
	"""
		Provides methods to deal with the MTL file, used by the importprecheck process.
	"""
	
	def __init__(self,FullPathName):
		self.MaterialContainerList = []
		self.MTLFilePath = FullPathName
		self.ActualMTLFileFN = None
		
	def CreateCollapseFile(self,TheCollapseFN):
		"""
			Creates a hash table of all the unique images in the material file.
			Any additional materials with the same image can be collapsed. Creates a file with 
			the pairs of material names that can be collapsed.
		"""
		
		UsedImageHash = {}
		cntr = 0
		WorkMTLContainer = None
		LineFMT = "%s^%s" + os.linesep
			
		CollapseFile = file(TheCollapseFN,'w')
		
		for MyMTLContainer in self.MaterialContainerList:
			
			if MyMTLContainer.MaterialType != "NOImage":
				
				if UsedImageHash.__contains__(MyMTLContainer.ImageName) == False:
					UsedImageHash[MyMTLContainer.ImageName] = MyMTLContainer
				else:
					WorkMTLContainer = UsedImageHash[MyMTLContainer.ImageName]
					if WorkMTLContainer.MaterialType == MyMTLContainer.MaterialType:
						cntr = cntr + 1
						CollapseFile.write(LineFMT % (MyMTLContainer.MaterialName,WorkMTLContainer.MaterialName))
		
		CollapseFile.close()
		
		print("Created %i possiable materials that can be collapsed") % cntr
		
	def DoAutoCorrect(self,dazflag):
		"""
			Find any materials that are a alpha map and ensure they have alpha set to 0.
			If this export is from DAZ adjust the hardness values.
		
		"""
		
		for MyMTLContainer in self.MaterialContainerList:
			if MyMTLContainer.MaterialType == "map_d" or MyMTLContainer.MaterialType == "map_D":
				self.__CorrectAlphaIfNeeded(MyMTLContainer)
		
		NewMTLFileFN = self.ActualMTLFileFN + "_New"
		print("Creating new mtl file %s") % NewMTLFileFN
		NewMTLFile = file(NewMTLFileFN,'w')
		
		# now that any alpha corrections are done write out the file. If this was a DAZ
		# export adjust any Ns lines at this time.
		
		for MyMTLContainer in self.MaterialContainerList:
			for WorkLine in MyMTLContainer.MTLFileLines:
				
				if dazflag == True:
					WorkDataList = WorkLine.split()
					if WorkDataList[0] == "Ns":
						dazhardness = float(WorkDataList[1])
						blenderhardness = int(dazhardness * 36.0)
						WorkLine = "Ns " + str(blenderhardness)
						
				NewMTLFile.write(WorkLine + os.linesep)
		
		NewMTLFile.close()
		
		try:
			os.remove(self.ActualMTLFileFN)
			os.rename(NewMTLFileFN,self.ActualMTLFileFN)
		except:
			myresult = sys.exc_info()
			print("Failed to rename material file:%s %s") % (str(myresult[0]),str(myresult[1]))
			
	def ParseMTLFile(self,TheMTLFileFN):
		"""
			Parse the mtl file, we are only interested in the image type of materials.
			Build a list of material names and the associated images.
			
		"""
		
		MyMTL = None
		
		print("Parsing mtl file %s") % TheMTLFileFN
		
		self.ActualMTLFileFN = TheMTLFileFN
		MyMTLFile = file(TheMTLFileFN,'r')
		
		for TheLine in MyMTLFile.readlines():
			WorkLine = string.strip(TheLine,'\r\n')
			WorkLineList = string.split(WorkLine)
			
			if len(WorkLine) == 0:
				continue

			if WorkLineList[0] == "#":
				continue
			
			if WorkLineList[0] == "newmtl":
				
				if MyMTL != None:
					
					if MyMTL.MaterialType is None:
						MyMTL.MaterialType = "NOImage"
						MyMTL.ImageName = "NOImage"
						
					self.MaterialContainerList.append(MyMTL)
					
				MyMTL = MTLContainer()
				MyMTL.MaterialName = WorkLineList[1]
				print("Now loading %s") % MyMTL.MaterialName
			
			WorkString = WorkLineList[0]
			
			if WorkString.startswith("map_") == True:
				MyMTL.MaterialType = WorkLineList[0]
				MyMTL.ImageName = WorkLineList[1]
				
			MyMTL.MTLFileLines.append(WorkLine)
			
		self.MaterialContainerList.append(MyMTL)
		
		MyMTLFile.close()
		
	def __CorrectAlphaIfNeeded(self,MyMTLContainer):
		"""
			Private method to set alpha to 0 if the autocorrect flag was set in the 
			importprecheck.py
		"""
		
		cntr = 0
		for TheLine in MyMTLContainer.MTLFileLines:
			WorkLine = string.split(TheLine)
			
			if WorkLine[0] == "d" and WorkLine[1] == "1":
				print("Material %s should be an alpha map but alpha is not zero, correcting it") % MyMTLContainer.MaterialName
				MyMTLContainer.MTLFileLines[cntr] = "d 0"
			
			cntr = cntr + 1
				
				