# CVS $Id: PoserImporter.py,v 1.3 2007/07/08 15:23:01 jay Exp $
#=========================================================================
# Main object to handle the import of a .obj file from poser or daz
# 
#  Copyright (C) 2006 Jay van der Meer
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#=========================================================================
import os
import string
import Blender
from FaceImporter import *
from MTLObject import *

class PoserImporter:
  """
    This is the main class that does an import of poser/daz object into blender.
    Properties are setup to hold the vertex, uv and other lines as they are read
    from the .obj file.
    
  """
  def __init__(self,TheOBJFN,TheBasePath,DefaultMeshName):
    self.VertexLines = []
    self.VTLines = []
    self.AllOtherLines = []
    self.otherrecordcnt = 0
    self.MyOBJFn = TheOBJFN
    self.MyBasePath = TheBasePath
    self.MyMTLObject = None
    self.MyDefaultMeshName = DefaultMeshName
    
  def __ParseMTLFile(self,TheMTLFN):
    """
      Private method to create the mtl object parser and request the object
      do the parse.
      
    """
    self.MyMTLObject = MTLObject(self.MyBasePath)
    MTLFn = self.MyBasePath + os.sep + TheMTLFN
    print("Doing parse of MTL file %s") % MTLFn
    self.MyMTLObject.ParseMTLFile(MTLFn)
    
  def ParseOBJFile(self):
    """
      Load all the lines of the obj file into the various lists, based on tag.
      
    """
    ablank = " "
    vt1 = ''
    vt2 = ''
    
    MyOBJFile = file(self.MyOBJFn,'r')
    
    for TheLine in MyOBJFile.readlines():
      CleanLine = TheLine.replace('\t',' ')
      CleanLine = string.strip(CleanLine,'\r\n')
      WorkLine = string.split(CleanLine)
      
      if len(WorkLine) == 0:
        continue
      
      if WorkLine[0] == "#":
        continue

      if WorkLine[0].startswith("mtllib") == True:
        if self.MyMTLObject is None:
          self.__ParseMTLFile(WorkLine[1].strip())
        continue
      
      if WorkLine[0] == "v":
        self.VertexLines.append(TheLine)
        continue
        
      if WorkLine[0] == "vt":
        vt1 = float(WorkLine[1])
        vt2 = float(WorkLine[2])
        self.VTLines.append((vt1,vt2))
        continue
      
      if WorkLine[0] == "vn":
        continue
      
      self.AllOtherLines.append(CleanLine)
    
    MyOBJFile.close()
    
  def DoImport(self):
    """
      Main method that controls the import. This method creates the vertex, faces and appends them
      into the blender mesh object
      
    """
    MyCurrentMaterialHash = {}
    MyFaceIMP = FaceImporter()
    MyFaceIMP.VTList = self.VTLines
    currentmatidx = -1
    materialindex = -1
    MyCurrentMaterialName = ''
    MyMesh = Blender.NMesh.New(self.MyDefaultMeshName)
    
    for TheLine in self.VertexLines:
      TheWords = string.split(TheLine)
      x = float(TheWords[1])
      y = float(TheWords[2])
      z = float(TheWords[3])
      MyMesh.verts.append(Blender.NMesh.Vert(x,y,z))
      
    for TheLine in self.AllOtherLines:
      TheWords = string.split(TheLine)
      
      if TheWords[0] == 'usemtl':
        
        MyCurrentMaterialName = TheWords[1]
        
        if MyCurrentMaterialHash.__contains__(MyCurrentMaterialName) == False:
          if self.MyMTLObject.MaterialObjectHash.__contains__(MyCurrentMaterialName) == True:
            materialindex = materialindex + 1
            print("Adding material %s to mesh as index %i") % (MyCurrentMaterialName,materialindex)
            MyCurrentMaterialHash[MyCurrentMaterialName] = materialindex
            MyMesh.addMaterial(self.MyMTLObject.MaterialObjectHash[MyCurrentMaterialName])
        
        currentmatidx = int(MyCurrentMaterialHash[MyCurrentMaterialName])
        
        continue
      
      if TheWords[0] == 'f':
        NewFace = MyFaceIMP.ProcessFaceRecord(TheWords,MyMesh)
        
        if NewFace != None:
          if currentmatidx != -1:
            NewFace.materialIndex = currentmatidx
          
          MyMesh.addFace(NewFace)
    
    MyObj = Blender.Object.New('Mesh')
    MyObj.name = self.MyDefaultMeshName + "_M"
    MyObj.link(MyMesh)
    MyScene = Blender.Scene.GetCurrent()
  
    for o in MyScene.objects:
      o.sel = 0
    

    MyScene.objects.link(MyObj)
    MyObj.sel = 1
    MyObj.Layers = MyScene.Layers
    