# CVS $Id: collapsematerials.py,v 1.5 2006/08/11 00:16:34 jay Exp $
#=========================================================================
# Program to do a find replace of selected materials in the .obj file. The materials to work with are 
# supplied in the collapse file. The format is <old-material>:<new material>
#
#  Copyright (C) 2006 Jay van der Meer
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#=========================================================================
import os
import string
import getopt
import sys


cmdline_parms = sys.argv[1:]
TheOBJFN = None
TheCollapseFile = None
TheNewOBJFN = None

# format of the translate table, key is the material to find, value is the replacement material
MaterialTranslateHash = {}

opts,args = getopt.getopt(cmdline_parms,"c:o:")

for option,parm in opts:
	if option == "-c":
		TheCollapseFile = parm
	
	if option == "-o":
		TheOBJFN = parm

if TheOBJFN is None or TheCollapseFile is None:
	print("Usage is collapsematerials.py -c <full-path-to-collapse-file> -o <full path to obj file>")
	print("Where -c points to the full path and name of the collapse file to write.")
	print("-o is the full path to the .obj file to process")
	sys.exit(8)
	
CollapseFile = file(TheCollapseFile,'r')

# Load the collapse file into a hash table. Key is the material to replace, value
# is the replacement material name.

for WorkLine in CollapseFile.readlines():

	WorkLine = string.strip(WorkLine,'\r\n')
	
	if len(WorkLine) == 0:
		continue

	WorkList = string.split(WorkLine,'^')
	MaterialTranslateHash[WorkList[0]] = WorkList[1]
	print("Adding translation of material %s to %s") % (WorkList[0],WorkList[1])
	
CollapseFile.close()
	
TheNewOBJFN = TheOBJFN + "_New"

OldOBJFile = file(TheOBJFN,'r',4096)
NewOBJFile = file(TheNewOBJFN,'w')
NewMatLineFMT = "usemtl %s" + os.linesep
NormalLineFMT = "%s" + os.linesep

# Read the obj file and replace any materials found in the hash table

for WorkLine in OldOBJFile.readlines():
	WorkLine = string.strip(WorkLine,'\r\n')
	WorkList = string.split(WorkLine)

	if len(WorkList) == 0:
		continue
	
	if WorkList[0] == "usemtl":
		if MaterialTranslateHash.__contains__(WorkList[1]) == True:
			MyNewMaterial = MaterialTranslateHash[WorkList[1]]
			print("Found occurance of %s, replaced with %s") % (WorkList[1],MyNewMaterial)
			NewOBJFile.write(NewMatLineFMT % MyNewMaterial)
		else:
			NewOBJFile.write(NormalLineFMT % WorkLine)
			
	else:
		NewOBJFile.write(NormalLineFMT % WorkLine)

OldOBJFile.close()
NewOBJFile.close()

try:
	os.remove(TheOBJFN)
	os.rename(TheNewOBJFN,TheOBJFN)
except:
	myresult = sys.exc_info()
	print("Failed to rename object file:%s %s") % (str(myresult[0]),str(myresult[1]))

sys.exit(0)


