# CVS $Id: importprecheck.py,v 1.4 2006/06/05 21:53:20 jay Exp $
#=========================================================================
# Run a pre check on a .obj file created in poser or daz.
# 
#  Copyright (C) 2006 Jay van der Meer
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#=========================================================================
import os
import string
import getopt
import sys
from ImportPreChecker import *
from MTLReWriter import *

cmdline_parms = sys.argv[1:]
TheOBJFN = None
TheCollapseFile = None
autocorrect = False
dazflag = False

opts,args = getopt.getopt(cmdline_parms,"c:o:ad")

for option,parm in opts:
	if option == "-c":
		TheCollapseFile = parm
	
	if option == "-o":
		TheOBJFN = parm
		
	if option == "-a":
		autocorrect = True
		
	if option == "-d":
		dazflag = True

if TheOBJFN is None or TheCollapseFile is None:
	print("Usage is importprecheck.py -c <full-path-to-collapse-file> -o <full path to obj file> [-a] [-d]")
	print("Where -c points to the full path and name of the collapse file to write.")
	print("-o is the full path to the .obj file to import")
	print("-a is optional autocorrect flag and -d indicates it an export from DAZ.")
	sys.exit(8)
	
# run a check on the obj file to find out how many materials are actually used.

MyBasePath,MyFN = os.path.split(TheOBJFN)
	
MyPreChecker = ImportPreChecker(TheOBJFN,TheCollapseFile)

matcnt = MyPreChecker.CheckMaterialLimit()
matcnt = matcnt + 1

MyMTLWriter = MTLReWriter(MyBasePath)

if len(MyBasePath) == 0:
	MTLFn = MyPreChecker.MTLLibFN
else:
	MTLFn = MyBasePath + os.sep + MyPreChecker.MTLLibFN

MyMTLWriter.ParseMTLFile(MTLFn)

# if autocorrect is true then make sure any image map_d has alpha set to 0

if autocorrect == True:
	if dazflag == True:
		print("Doing autocorrect of mtl file, including hardness adjustments for DAZ")
	else:
		print("Doing autocorrect of mtl file.")
		
	MyMTLWriter.DoAutoCorrect(dazflag)

if matcnt > 16:
	print("OBJ file uses %i materials only 16 materials are allowed in blender. Creating list of materials that can be collapsed.") % matcnt
	print("Either export some objects separately or run collapsematerials.py with the output from this process.")
	MyMTLWriter.CreateCollapseFile(TheCollapseFile)
else:
	print("OBJ files uses less than 16 materials, ready for import.")
	
sys.exit(0)
